<?php

namespace Bixo\Announcement\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Announcement\Forms\Announcement as AnnouncementForm;
use Bixo\Announcement\Http\Requests\AnnouncementResourceRequest;
use Bixo\Announcement\Http\Resources\AnnouncementResource;
use Bixo\Announcement\Http\Resources\AnnouncementsCollection;
use Bixo\Announcement\Models\Announcement;
use Bixo\Announcement\Actions\AnnouncementAction;
use Bixo\Announcement\Actions\AnnouncementActions;


/**
 * Resource controller class for announcement.
 */
class AnnouncementResourceController extends BaseController
{

    /**
     * Initialize announcement resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = AnnouncementForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.announcement.modules'), 'announcement', guard_url('announcement'));
            return $next($request);
        });
    }

    /**
     * Display a list of announcement.
     *
     * @return Response
     */
    public function index(AnnouncementResourceRequest $request)
    {
        $request = $request->all();
        $page = AnnouncementActions::run('paginate', $request);

        $data = new AnnouncementsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('announcement::announcement.names'))
            ->view('announcement::announcement.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display announcement.
     *
     * @param Request $request
     * @param Model   $announcement
     *
     * @return Response
     */
    public function show(AnnouncementResourceRequest $request, Announcement $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AnnouncementResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('announcement::announcement.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('announcement::announcement.show')
            ->output();
    }

    /**
     * Show the form for creating a new announcement.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AnnouncementResourceRequest $request, Announcement $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AnnouncementResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('announcement::announcement.name'))
            ->view('announcement::announcement.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new announcement.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AnnouncementResourceRequest $request, Announcement $model)
    {
        try {
            $request = $request->all();
            $model = AnnouncementAction::run('store', $model, $request);
            $data = new AnnouncementResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('announcement::announcement.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('announcement/announcement/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/announcement/announcement'))
                ->redirect();
        }

    }

    /**
     * Show announcement for editing.
     *
     * @param Request $request
     * @param Model   $announcement
     *
     * @return Response
     */
    public function edit(AnnouncementResourceRequest $request, Announcement $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AnnouncementResource($model);
        // return view('announcement::announcement.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('announcement::announcement.name'))
            ->view('announcement::announcement.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the announcement.
     *
     * @param Request $request
     * @param Model   $announcement
     *
     * @return Response
     */
    public function update(AnnouncementResourceRequest $request, Announcement $model)
    {
        try {
            $request = $request->all();
            $model = AnnouncementAction::run('update', $model, $request);
            $data = new AnnouncementResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('announcement::announcement.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('announcement/announcement/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('announcement/announcement/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the announcement.
     *
     * @param Model   $announcement
     *
     * @return Response
     */
    public function destroy(AnnouncementResourceRequest $request, Announcement $model)
    {
        try {

            $request = $request->all();
            $model = AnnouncementAction::run('destroy', $model, $request);
            $data = new AnnouncementResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('announcement::announcement.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('announcement/announcement/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('announcement/announcement/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
