<?php

namespace Bixo\Announcement\Http\Controllers;

use Exception;
use Bixo\Announcement\Actions\AnnouncementWorkflow;
use Bixo\Announcement\Http\Requests\AnnouncementWorkflowRequest;
use Bixo\Announcement\Http\Resources\AnnouncementResource;
use Bixo\Announcement\Models\Announcement;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class AnnouncementWorkflowController extends BaseController
{
    /**
     * Action controller function for announcement.
     *
     * @param Model $announcement
     * @param action next action for the announcement.
     *
     * @return Response
     */
    public function __invoke(AnnouncementWorkflowRequest $request, Announcement $announcement, $trasition)
    {
        try {
            $request = $request->all();
            $announcement = AnnouncementWorkflow::run($trasition, $announcement, $request);
            $data = new AnnouncementResource($announcement);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('announcement::announcement.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('announcement/announcement/' . $announcement->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('announcement/announcement/' . $announcement->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the announcement and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $announcement
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $announcement, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $announcement = Announcement::findBySignedId($announcement);
        $transitions = $this->transitions($announcement, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('announcement::announcement.name') . ' ' . trans('Approvals'))
            ->view('announcement::public.announcement.workflow')
            ->layout('mobile')
            ->data(compact('announcement', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the announcement.
     *
     * @param PublicRequest $request
     * @param string $announcement
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $announcement, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $announcement = Announcement::findBySignedId($announcement);
        $user = user();
        $announcement = AnnouncementWorkflow::run($transition, $announcement, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($announcement, $user)
    {
        $transitions = [];
        foreach ($announcement->workflow()->transitions($announcement) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.announcement.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'announcement' => $announcement->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $announcement->workflow()->form($value);
            $array['label'] = trans('announcement::announcement.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
